package com.xkrs.common.tools;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.core.JsonGenerator;
import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.*;
import com.fasterxml.jackson.datatype.jsr310.JavaTimeModule;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * 序列化工具
 * @author tajochen
 */
public class JackSonTool {

    private static final Logger logger = LoggerFactory.getLogger(JackSonTool.class);

    private JackSonTool() {
    }

    private static final ObjectMapper OBJECT_MAPPER = new ObjectMapper();

    static {
        OBJECT_MAPPER.setSerializationInclusion(JsonInclude.Include.NON_NULL);
        OBJECT_MAPPER.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);
        OBJECT_MAPPER.configure(SerializationFeature.FAIL_ON_EMPTY_BEANS, false);
        OBJECT_MAPPER.configure(JsonParser.Feature.ALLOW_COMMENTS, true);
        OBJECT_MAPPER.configure(JsonParser.Feature.ALLOW_SINGLE_QUOTES, true);
        OBJECT_MAPPER.configure(JsonParser.Feature.ALLOW_UNQUOTED_FIELD_NAMES, true);
        OBJECT_MAPPER.getSerializerProvider().setNullValueSerializer(new JsonSerializer<>() {
            @Override
            public void serialize(Object o, JsonGenerator jsonGenerator, SerializerProvider serializerProvider) throws IOException {
                jsonGenerator.writeString("");
            }
        });
        // 解决jackson无法反序列化LocalDateTime的问题,引入jsr310标准
        JavaTimeModule javaTimeModule = new JavaTimeModule();
        OBJECT_MAPPER.registerModule(javaTimeModule);
        OBJECT_MAPPER.disable(SerializationFeature.WRITE_DATES_AS_TIMESTAMPS);
    }

    /**
     * 序列化
     * @param obj
     * @param <T>
     * @return
     */
    public static <T> String encode(T obj) {
        if (Objects.isNull(obj)) {
            return null;
        }
        try {
            return OBJECT_MAPPER.writeValueAsString(obj);
        } catch (Exception e) {
            logger.error("json encode error, obj={}", obj, e);
            return null;
        }
    }

    /**
     * 反序列化
     * @param json
     * @param valueType
     * @param <T>
     * @return
     */
    public static <T> T decode(String json, Class<T> valueType) {
        if (!json.isEmpty() && !Objects.isNull(valueType)) {
            try {
                return OBJECT_MAPPER.readValue(json, valueType);
            } catch (Exception e) {
                logger.error("json decode fail,jsonString={}, type={}", json, valueType.getName(), e);
            }
        }
        return null;
    }

    /**
     * 反序列化成list
     * @param <T>
     * @param json
     * @param clazz
     * @return
     */
    public static <T> List decode2List(String json, Class<T> clazz) {
        if (!json.isEmpty() && !Objects.isNull(clazz)) {
            try {
                return OBJECT_MAPPER.readValue(json, OBJECT_MAPPER.getTypeFactory().constructCollectionType(List.class, clazz));
            } catch (Exception var3) {
                logger.error("json decode2list fail,json={},classType={}", json, clazz.getName(), var3);
            }
        }
        return null;
    }

    /**
     * 字符串转换为 Map<String, Object>
     *
     * @param jsonString
     * @return
     * @throws Exception
     */
    public static <T> Map json2map(String jsonString) {

        try {
            return OBJECT_MAPPER.readValue(jsonString, Map.class);
        } catch (JsonProcessingException e) {
            e.printStackTrace();
            return null;
        }
    }

    /**
     * 字符串转换为 Map<String, T>
     */
    public static <T> Map<String, T> json2map(String jsonString, Class<T> clazz) {
        Map<String, T> map = null;
        try {
            map = OBJECT_MAPPER.readValue(jsonString, new TypeReference<Map<String, T>>() {
            });
        } catch (JsonProcessingException e) {
            e.printStackTrace();
            return null;
        }
        Map<String, T> result = new HashMap<String, T>(128);
        for (Map.Entry<String, T> entry : map.entrySet()) {
            result.put(entry.getKey(), OBJECT_MAPPER.convertValue(entry.getValue(), clazz));
        }
        return result;
    }

    /**
     * 将一个JSON字符串转换为T对象
     * @param json
     * @param typeReference
     * @return
     */
    public static <T> T toObject(String json, TypeReference<T> typeReference) {
        try {
            return OBJECT_MAPPER.readValue(json, typeReference);
        } catch (Exception e) {
            throw new RuntimeException("failed to convert string to object", e);
        }
    }

    /**
     * map转clazz对象
     * @param map
     * @param clazz
     * @param <T>
     * @return
     */
    public static <T> T mapToObject(Map map, Class<T> clazz) {
        String json = null;
        try {
            json = OBJECT_MAPPER.writeValueAsString(map);
            return OBJECT_MAPPER.readValue(json, clazz);
        } catch (Exception e) {
            logger.error("json map2object fail,map={},ClassName={}", json, clazz.getName());
            return null;
        }
    }

}
